/********************************** (C) COPYRIGHT *******************************
 * File Name          : main.c
 * Author             : WCH
 * Version            : V1.0.0
 * Date               : 2020/04/30
 * Description        : Main program body.
 * Copyright (c) 2021 Nanjing Qinheng Microelectronics Co., Ltd.
 * SPDX-License-Identifier: Apache-2.0
 *******************************************************************************/
#include "ch32v30x.h"
#include "string.h"
#include "rtthread.h"
#include <rthw.h>
#include "drivers/pin.h"
#include "WCHNET.h"
#include "eth_driver.h"
#include "net_config.h"
/* Global typedef */

/* Global define */

/* LED0ͨrtpinӿ  */
#define LED0_PIN  18   //PC3

/* Global Variable */

/*********************************************************************
 * @fn      LED1_BLINK_INIT
 *
 * @brief   LED1ֱͨӵõײ
 *
 * @return  none
 */
void LED1_BLINK_INIT(void) {
    GPIO_InitTypeDef GPIO_InitStructure = { 0 };
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_GPIOA, ENABLE);
    GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
    GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
    GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
    GPIO_Init(GPIOA, &GPIO_InitStructure);
}

u8 MACAddr[6] = {0x08,0x01,0x01,0x01,0x02,0x03}; /* MACַ */
u8 IPAddr[4] = {192,168,10,30}; /* IPַ */
u8 GWIPAddr[4] = {192,168,10,1}; /*  */
u8 IPMask[4] = {255,255,0,0}; /*  */

/* ñ */
u8 SocketId, SocketIdForListen, RecvBuffer[RECE_BUF_LEN];
u8 SocketRecvBuf[WCHNET_MAX_SOCKET_NUM][RECE_BUF_LEN]; /* socketջ */
u8 flag = 0;
u16 DESPORT, SRCPORT;
u8 socket; /* ҳͨѶнsocket*/

int timer_sample(void);

#define printf rt_kprintf

void WCHNET_DataLoopback(u8 id)
{
    u8 i;
    u32 len;
    u32 endAddr = SocketInf[id].RecvStartPoint + SocketInf[id].RecvBufLen;

    if ((SocketInf[id].RecvReadPoint + SocketInf[id].RecvRemLen) > endAddr) {
        len = endAddr - SocketInf[id].RecvReadPoint;
    }
    else {
        len = SocketInf[id].RecvRemLen;
    }
    i = WCHNET_SocketSend(id, (u8 *) SocketInf[id].RecvReadPoint, &len);
    if (i == WCHNET_ERR_SUCCESS) {
        WCHNET_SocketRecv(id, NULL, &len);
    }
}
/*********************************************************************
 * @fn      WCHNET_HandleSockInt
 *
 * @brief   Socket Interrupt Handle
 *
 * @return  none
 */

void WCHNET_HandleSockInt(u8 socketid, u8 initstat) {
    u32 len;

    if (initstat & SINT_STAT_RECV) /* socketж*/
    {
        WCHNET_DataLoopback(socketid);
//        len = WCHNET_SocketRecvLen(socketid, NULL); /* ȡsocketݳ  */
//        //printf("WCHNET_SocketRecvLen %d  socket id %d\r\n", len, socketid);
//        if (len) {
//            WCHNET_SocketRecv(socketid, RecvBuffer, &len); /* յ󱣴RecvBuffer[]*/
//            flag = 1;
//            socket = socketid;
//        }
    }
    if (initstat & SINT_STAT_CONNECT) /* socketӳɹж*/
    {
        WCHNET_ModifyRecvBuf(socketid, (u32) SocketRecvBuf[socketid],
                RECE_BUF_LEN);
        printf("TCP Connect Success\r\n");
    }
    if (initstat & SINT_STAT_DISCONNECT) /* socketӶϿж*/
    {
        printf("TCP Disconnect\r\n");
    }
    if (initstat & SINT_STAT_TIM_OUT) /* socketӳʱж*/
    {
        printf("TCP Timeout\r\n");
        //WCHNET_CreatCfgSocket(Port_CfgBuf->mode, Port_CfgBuf->des_ip, DESPORT, SRCPORT);
    }
}

/*********************************************************************
 * @fn      WCHNET_HandleGlobalInt
 *
 * @brief   Global Interrupt Handle
 *
 * @return  none
 */
void WCHNET_HandleGlobalInt(void) {
    u8 initstat;
    u16 i;
    u8 socketinit;

    initstat = WCHNET_GetGlobalInt(); /* ȡȫжϱ־*/
    if (initstat & GINT_STAT_UNREACH) /* ɴж */
    {
        printf("GINT_STAT_UNREACH\r\n");
    }
    if (initstat & GINT_STAT_IP_CONFLI) /* IPͻж */
    {
        printf("GINT_STAT_IP_CONFLI\r\n");
    }
    if (initstat & GINT_STAT_PHY_CHANGE) /* PHY״̬仯ж */
    {
        i = WCHNET_GetPHYStatus(); /* ȡPHY״̬*/
        if (i & PHY_Linked_Status)
            printf("PHY Link Success\r\n");
    }
    if (initstat & GINT_STAT_SOCKET) {
        for (i = 0; i < WCHNET_MAX_SOCKET_NUM; i++) {
            socketinit = WCHNET_GetSocketInt(i);
            if (socketinit)
                WCHNET_HandleSockInt(i, socketinit);
        }
    }
}

void mStopIfError(u8 iError) {
    if (iError == WCHNET_ERR_SUCCESS)
        return;                                                 /* ɹ */
    printf("Error: %02X\r\n", (u16) iError);                    /* ʾ */
}

/*********************************************************************
 * @fn      WCHNET_CreatTcpSocketListen
 *
 * @brief   Create TCP Socket for Listening
 *
 * @return  none
 */
#define HTTP_SERVER_PORT          80
void WCHNET_CreatTcpSocketListen(void) {
    u8 i;
    SOCK_INF TmpSocketInf;

    memset((void *) &TmpSocketInf, 0, sizeof(SOCK_INF));            /* ڲὫ˱ƣýʱȫ */
    TmpSocketInf.SourPort = HTTP_SERVER_PORT;                       /* Դ˿ */
    TmpSocketInf.ProtoType = PROTO_TYPE_TCP;                        /* socket */
    i = WCHNET_SocketCreat(&SocketIdForListen, &TmpSocketInf);      /* socketصsocketSocketIdForListen*/
    printf("SocketIdForListen %d\r\n", SocketIdForListen);
    mStopIfError(i);                                                /*  */
    i = WCHNET_SocketListen(SocketIdForListen);                     /* TCP */
    mStopIfError(i);                                                /*  */
}


void TIM2_Init(void) {
    TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure = { 0 };

    RCC_APB1PeriphClockCmd(RCC_APB1Periph_TIM2, ENABLE);

    TIM_TimeBaseStructure.TIM_Period = SystemCoreClock / 1000000 - 1;
    TIM_TimeBaseStructure.TIM_Prescaler = WCHNETTIMERPERIOD * 1000 - 1;
    TIM_TimeBaseStructure.TIM_ClockDivision = 0;
    TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
    TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);
    TIM_ITConfig(TIM2, TIM_IT_Update, ENABLE);

    TIM_Cmd(TIM2, ENABLE);
    TIM_ClearITPendingBit(TIM2, TIM_IT_Update);
    NVIC_EnableIRQ(TIM2_IRQn);
}
/*********************************************************************
 * @fn      main
 *
 * @brief   mainֻһ֮߳һ֮⻹tshell,idle
 *          mainֻһLED˸main̵߳עrtthread_startupУtshellʹ˴
 *          жϣжջ߳ջʹ÷ֿ.
 *
 * @return  none
 */
int main(void) {
    int i = 0;
    rt_kprintf("\r\n MCU: CH32V307\r\n");
    rt_kprintf(" SysClk: %dHz\r\n", SystemCoreClock);
    rt_kprintf(" www.wch.cn\r\n");
    rt_kprintf("net version:%x\n", WCHNET_GetVer());

    while(0)
    {
        ;
    }

    rt_kprintf("IP:");
    for (i = 0; i < 4; i++)
    {
        rt_kprintf("%d.", IPAddr[i]);
    }
    rt_kprintf("\r\n");

    //TIM2_Init();

    i = ETH_LibInit(IPAddr, GWIPAddr, IPMask, MACAddr);                        //̫ʼ
    mStopIfError(i);
    WCHNET_CreatTcpSocketListen();

    LED1_BLINK_INIT();
    GPIO_ResetBits(GPIOA, GPIO_Pin_0);
    timer_sample();

    while(1)
    {
        WCHNET_MainTask(); //̫Ҫѭ
        if(WCHNET_QueryGlobalInt()) //ѯ̫ȫжϣжϣȫжϴ
        {
            WCHNET_HandleGlobalInt();
        }
//         GPIO_SetBits(GPIOA,GPIO_Pin_0);
//         rt_thread_mdelay(500);
//         GPIO_ResetBits(GPIOA,GPIO_Pin_0);
//         rt_thread_mdelay(500);
    }

}

/*********************************************************************
 * @fn      led
 *
 * @brief   ʹӿڲI/O
 *
 * @return  none
 */
int led(void) {
    rt_uint8_t count;

    rt_pin_mode(LED0_PIN, PIN_MODE_OUTPUT);
    printf("led_SP:%08x\r\n", __get_SP());
    for (count = 0; count < 10; count++) {
        rt_pin_write(LED0_PIN, PIN_LOW);
        rt_kprintf("led on, count : %d\r\n", count);
        rt_thread_mdelay(500);

        rt_pin_write(LED0_PIN, PIN_HIGH);
        rt_kprintf("led off\r\n");
        rt_thread_mdelay(500);
    }
    return 0;
}

MSH_CMD_EXPORT(led, led sample by using I/O drivers);

/* ʱĿƿ */
static rt_timer_t timer1;
/* ʱ1ʱ */
static void timeout1(void *parameter)
{
    WCHNET_TimeIsr(10);
}

int timer_sample(void)
{
/* ʱ1  ڶʱ */
   timer1 = rt_timer_create("timer1", timeout1,
                            RT_NULL, 10,
                            RT_TIMER_FLAG_PERIODIC);
   /* ʱ1 */
   if (timer1 != RT_NULL)
   {
       rt_timer_start(timer1);
   }
}

